<?php
/*
Copyright (©) 2003-2013 Teus Benschop.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


class Database_Ipc
{
  private static $instance;
  private function __construct() {
  } 
  public static function getInstance() 
  {
    if (empty (self::$instance)) {
      self::$instance = new Database_Ipc();
    }
    return self::$instance;
  }


  public function trim ()
  {
    $database_instance = Database_Instance::getInstance();
    $database_instance->runQuery ("DELETE FROM ipc WHERE user = '';");
  }
  
  
  public function optimize ()
  {
    $database_instance = Database_Instance::getInstance();
    $database_instance->runQuery ("REPAIR TABLE ipc;");
    $database_instance->runQuery ("OPTIMIZE TABLE ipc;");
  }


  public function storeMessage ($user, $channel, $command, $message)
  {
    $user    = Database_SQLInjection::no ($user);
    $channel = Database_SQLInjection::no ($channel);
    $command = Database_SQLInjection::no ($command);
    $message = Database_SQLInjection::no ($message);
    $database_instance = Database_Instance::getInstance();
    if ($channel == "") {
      $query = "DELETE FROM ipc WHERE user = '$user' AND channel = '' AND command = '$command';";
      $database_instance->runQuery ($query);
    }
    $query = "INSERT INTO ipc VALUES (NULL, '$user', '$channel', '$command', '$message');";
    $database_instance->runQuery ($query);
  }


  /**
  * Retrieves a message if there is any.
  * Returns NULL if there was nothing, 
  * else an associated array with keys and values for "id", "channel", "command" and "message".
  */
  public function retrieveMessage ($id, $user, $channel, $command)
  {
    $id = Database_SQLInjection::no ($id);
    $channel = Database_SQLInjection::no ($channel);
    $command = Database_SQLInjection::no ($command);
    $database_instance = Database_Instance::getInstance();
    $query = "SELECT id, channel, command, message FROM ipc WHERE id > $id AND (channel = '$channel' OR channel = '') AND (user = '$user' OR user = '') AND command = '$command' ORDER BY id DESC;";
    $result = $database_instance->runQuery ($query);
    $values = NULL;
    if ($result->num_rows > 0) {
      $values = $result->fetch_assoc();
    }
    return $values;
  }

  
  public function deleteMessage ($id)
  {
    $id = Database_SQLInjection::no ($id);
    $database_instance = Database_Instance::getInstance();
    $query = "DELETE FROM ipc WHERE id = $id;";
    $database_instance->runQuery ($query);
  }


  public function getFocus ()
  {
    $session_logic = Session_Logic::getInstance ();
    $user = $session_logic->currentUser ();
    $user = Database_SQLInjection::no ($user);
    $message = array ();
    $database_instance = Database_Instance::getInstance();
    $query = "SELECT message FROM ipc WHERE command = 'focus' AND user = '$user' ORDER BY id DESC;";
    $result = $database_instance->runQuery ($query);
    if ($result->num_rows > 0) {
      $message = $result->fetch_row();
      return $message[0];
    }
    // No focus found: Return Genesis 1:1.
    return "1.1.1";
  }
  

  public function getNote ()
  {
    $session_logic = Session_Logic::getInstance ();
    $user = $session_logic->currentUser ();
    $user = Database_SQLInjection::no ($user);
    $database_instance = Database_Instance::getInstance();
    $query = "SELECT id, message FROM ipc WHERE user = '$user' AND command = 'opennote' ORDER BY id DESC;";
    $result = $database_instance->runQuery ($query);
    if ($result->num_rows > 0) {
      return $result->fetch_assoc ();
    }
    return NULL;
  }


  public function getAlive ()
  {
    $session_logic = Session_Logic::getInstance ();
    $user = $session_logic->currentUser ();
    $user = Database_SQLInjection::no ($user);
    $database_instance = Database_Instance::getInstance();
    $query = "SELECT message FROM ipc WHERE user = '$user' AND command = 'notesalive' ORDER BY id DESC;";
    $result = $database_instance->runQuery ($query);
    if ($result->num_rows > 0) {
      $alive = $result->fetch_row ();
      return $alive [0];
    }
    return false;
  }


}



?>
