<?php
/*
Copyright (©) 2003-2013 Teus Benschop.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
  
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


require_once ("../bootstrap/bootstrap.php");


// The script runs through the cli Server API only.
if (php_sapi_name () != "cli") {
  die;
}


// Change the user id for running this script.
$pwnam = posix_getpwnam ("www-data");
posix_setuid ($pwnam['uid']);
posix_setgid ($pwnam['gid']);
posix_seteuid ($pwnam['uid']);
posix_setegid ($pwnam['gid']);


ignore_user_abort (true);
set_time_limit (0);


include ("paths/paths.php");


$config_general = Database_Config_General::getInstance ();
$database_logs = Database_Logs::getInstance ();
$timer_logger = new Timer_Logger ();
//$processUser = posix_getpwuid(posix_geteuid());
//$processUser = $processUser['name'];


$current_timestamp = time ();


// CPU-intensive actions run at night.
// This keeps the site more responsive during the day.
$hour = date ('G');
$minute = date ('i');


// Every minute check on processes that are stuck.
$timer_logger->killExpiredPIDs ();
// Every minute deal with any log files that were used for the scripts.
$timer_logger->handleUsedLogFiles ();


// Every minute send out any queued mail.
$timer_logger->startProcess (dirname (__FILE__), "php", "mailer.php", NULL, NULL, false);


// Check for new mail every five minutes.
// Do not check more often with gmail else the account may be shut down.
if (($minute % 5) == 0) {
  $timer_logger->startProcess (dirname (__FILE__), "php", "receiver.php", NULL, NULL, false);
}


// The order for running the following nightly scripts is important.
// Any of those scripts may influence the subsequent ones.
// The order is such that all information generated is as recent as possible.
// More important tasks are done first, and the less important ones at the end.
// This leads to an order as visible in the code below.


// Sending and receiving Bibles to and from the git repository.
// On a production website running on an inexpensive virtual private server 
// with 512 Mbyte of memory and a fast network connection, 
// sending and receiving two Bibles takes more than 15 minutes when there are many changes.
$sendreceive = false;
if (($hour == 0) && ($minute == 0)) $sendreceive = true;
if ((($minute % 5) == 0) && $config_general->getRepeatSendReceive ()) $sendreceive = true;
if ($sendreceive) {
  if (!SendReceive_Logic::isRunning ()) {
    SendReceive_Logic::start ();
  } else {
    $database_logs->log ("Intending to Send/Receive, but the process is already running");
  }
}


// Deal with the changes in the Bible made per user.
// It should run before the general changes to influence the order of the notifications in the GUI.
if (($hour == 0) && ($minute == 20)) {
  $timer_logger->startProcess (dirname (__FILE__), "php", "userchanges.php", NULL, NULL, true);
}


// Deal with notifications for the daily changes in the Bibles.
// This takes a few minutes on a production machine with two Bibles and changes in several chapters.
if (($current_timestamp >= $config_general->getTimerDiff ()) || (($hour == 0) && ($minute == 25))) {
  $config_general->setTimerDiff ($current_timestamp + 100000);
  $timer_logger->startProcess (dirname (__FILE__), "php", "changes.php", NULL, NULL, true);
}


// Comb through the git log and add the change to the history.
if (($hour == 0) && ($minute == 30)) {
  $directory = dirname (dirname (__FILE__)) . "/changes";
  $timer_logger->startProcess ($directory, "php", "git.php", NULL, NULL, true);
  unset ($directory);
}


// Run the checks on the Bibles.
// This takes 15 minutes on a production machine with two Bibles.
if (($current_timestamp >= $config_general->getTimerChecks ()) || (($hour == 0) && ($minute == 30))) {
  $config_general->setTimerChecks ($current_timestamp + 100000);
  $timer_logger->startProcess (dirname (__FILE__), "php", "checks.php", NULL, NULL, true);
}


// Database maintenance and trimming.
// It takes a few minutes on a production machine.
if (($hour == 0) && ($minute == 50)) {
  $timer_logger->startProcess (dirname (__FILE__), "php", "database.php", NULL, NULL, true);
}


// Create a backup, so that the backup contains the most recent information
// after the previous tasks have been done.
if (($current_timestamp >= $config_general->getTimerBackup ()) || (($hour == 0) && ($minute == 55))) {
  $config_general->setTimerBackup ($current_timestamp + 100000);
  $timer_logger->startProcess (dirname (__FILE__), "php", "backup.php", NULL, NULL, true);
}


// Email statistics to the users.
if (($hour == 1) && ($minute == 0)) {
  $timer_logger->startProcess (dirname (__FILE__), "php", "statistics.php", NULL, NULL, true);
}


// Sprint burndown.
// It runs at midnight, but also in the morning and afternoon.
// The script itself determines what to do at various times of the day,
// or days of the week or month.
$burndown = false;
if (($hour == 1) && ($minute == 5)) $burndown = true;
if (($hour == 10) && ($minute == 0)) $burndown = true;
if (($hour == 14) && ($minute == 0)) $burndown = true;
if ($burndown) {
  Sprint_Logic::burndown (false);
}
unset ($burndown);


// A flag is set upon installation, so it (re)generates the stylesheets once.
$create_css_flag = "$localStatePath/$location/create_css_flag";
if (file_exists ($create_css_flag)) {
  unlink ($create_css_flag);
  Styles_Sheets::create_all ();
}


// Export the Bibles to the various output formats.
// This may take an hour on a production machine.
if (($current_timestamp >= $config_general->getTimerExports ()) || (($hour == 1) && ($minute == 10))) {
  $config_general->setTimerExports ($current_timestamp + 100000);
  $timer_logger->startProcess (dirname (__FILE__), "php", "exports.php", NULL, NULL, true);
}


// Clear the past sessions.
if (($hour == 2) && ($minute == 0)) {
  $sessions_path = dirname (__FILE__) . "/../session";
  $parameters = array ($sessions_path);
  $timer_logger->startProcess (dirname (__FILE__), "php", "sessions.php", $parameters, NULL, true);
}


?>
